/*------------------------------------------------------------------------------
--  This file is a part of the libuc, microcontroler library
--  Copyright (C) 2011, Alexis Jeandet
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
-------------------------------------------------------------------------------
--                 Author : Alexis Jeandet
--                   Mail : alexis.jeandet@gmail.com
-------------------------------------------------------------------------------*/

#include "LPC17xx.h"
#include "bsp.h"
#include "core.h"
#include "uart.h"
#include "libucstrings.h"
#include <nxplib.h>
#include <arm_math.h>
#define __delay(t) for(volatile int i=0;i<t;i++);

#define _ADC_INT		ADC_ADINTEN5
#define _ADC_CHANNEL		ADC_CHANNEL_5


int main (void) 
{
	PINSEL_CFG_Type PinCfg;
	uint32_t adc_value, tmp;
	bsp_init();
	uartsetbaudrate(UART0,38400);
	PinCfg.Funcnum = 3;
	PinCfg.OpenDrain = 0;
	PinCfg.Pinmode = 0;
	PinCfg.Pinnum = 31;
	PinCfg.Portnum = 1;
	PINSEL_ConfigPin(&PinCfg);
	ADC_Init(LPC_ADC, 200000);
	ADC_IntConfig(LPC_ADC,_ADC_INT,DISABLE);
	ADC_ChannelCmd(LPC_ADC,_ADC_CHANNEL,ENABLE);
	int i = 0;
	
	q15_t vector1[10]={0x8001,0x8002,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000};
	q15_t vector2[10]={0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000};
	q15_t vector3[10];
	arm_mult_q15(vector1,vector2,vector3,10);
	for(int i = 0;i<10;i++)
	{
		libucprintf("vector3[%d] = 0x%x \n\r",i,vector3[i]);
	}
	while(1)
	{
		LED1_OFF;
		LED2_ON;
		LED3_OFF;
		LED4_ON;
		LED5_OFF;
		LED6_ON;
		LED7_OFF;
		LED8_ON;
		ADC_StartCmd(LPC_ADC,ADC_START_NOW);
		//Wait conversion complete
		while (!(ADC_ChannelGetStatus(LPC_ADC,_ADC_CHANNEL,ADC_DATA_DONE)));
		adc_value = ADC_ChannelGetData(LPC_ADC,_ADC_CHANNEL);
		//Display the result of conversion on the UART0
		//uartputs(UART2,"Hello World\n\r");
		libucprintf("test %d \r",adc_value);
		__delay(1024*16);		
		LED1_ON;
		LED2_OFF;
		LED3_ON;
		LED4_OFF;
		LED5_ON;
		LED6_OFF;
		LED7_ON;
		LED8_OFF;
		//uartputs(UART0,"Hello World\n\r");
		__delay(1024*16);
	}
        ADC_DeInit(LPC_ADC);
	return 0;
  
}

